@@ -20,12 +20,17 @@
 #ifndef DECK_H
 #define DECK_H
 
+#include <math.h>
+
+#include "cues.h"
 #include "device.h"
 #include "listing.h"
 #include "player.h"
 #include "realtime.h"
 #include "timecoder.h"
 
+#define NO_PUNCH (HUGE_VAL)
+
 struct deck {
     struct device device;
     struct timecoder timecoder;
@@ -34,6 +39,16 @@ struct deck {
 
     struct player player;
     const struct record *record;
+    struct cues cues;
+
+    /* Punch */
+
+    double punch;
+
+    /* A controller adds itself here */
+
+    size_t ncontrol;
+    struct controller *control[4];
 };
 
 int deck_init(struct deck *deck, struct rt *rt);
@@ -45,5 +60,9 @@ void deck_load(struct deck *deck, struct record *record);
 
 void deck_recue(struct deck *deck);
 void deck_clone(struct deck *deck, const struct deck *from);
+void deck_unset_cue(struct deck *deck, unsigned int label);
+void deck_cue(struct deck *deck, unsigned int label);
+void deck_punch_in(struct deck *d, unsigned int label);
+void deck_punch_out(struct deck *d);
 
 #endif
