@@ -22,17 +22,21 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/mman.h> /* mlockall() */
 
 #include <SDL.h> /* may override main() */
 
 #include "alsa.h"
+#include "controller.h"
 #include "deck.h"
 #include "device.h"
+#include "dicer.h"
 #include "interface.h"
 #include "jack.h"
 #include "library.h"
 #include "oss.h"
 #include "realtime.h"
+#include "thread.h"
 #include "rig.h"
 #include "timecoder.h"
 #include "track.h"
@@ -44,6 +48,7 @@
 #define DEFAULT_ALSA_BUFFER 8 /* milliseconds */
 
 #define DEFAULT_RATE 44100
+#define DEFAULT_PRIORITY 80
 
 #define DEFAULT_IMPORTER EXECDIR "/xwax-import"
 #define DEFAULT_SCANNER EXECDIR "/xwax-scan"
@@ -56,18 +61,28 @@ char *banner = "xwax " VERSION \
 
 static void usage(FILE *fd)
 {
-    fprintf(fd, "Usage: xwax [<options>]\n\n"
+    fprintf(fd, "Usage: xwax [<options>]\n\n");
+
+    fprintf(fd, "Program-wide options:\n"
+      "  -k             Lock real-time memory into RAM\n"
+      "  -q <n>         Real-time priority (0 for no priority, default %d)\n"
+      "  -h             Display this message to stdout and exit\n\n",
+      DEFAULT_PRIORITY);
+
+    fprintf(fd, "Music library options:\n"
       "  -l <path>      Location to scan for audio tracks\n"
       "  -p <path>      Ordered playlist for audio tracks\n"
+      "  -s <program>   Library scanner (default '%s')\n\n",
+      DEFAULT_SCANNER);
+
+    fprintf(fd, "Deck options:\n"
       "  -t <name>      Timecode name\n"
       "  -33            Use timecode at 33.3RPM (default)\n"
       "  -45            Use timecode at 45RPM\n"
       "  -c             Protect against certain operations while playing\n"
       "  -u             Allow all operations when playing\n"
-      "  -i <program>   Importer (default '%s')\n"
-      "  -s <program>   Library scanner (default '%s')\n"
-      "  -h             Display this message\n\n",
-      DEFAULT_IMPORTER, DEFAULT_SCANNER);
+      "  -i <program>   Importer (default '%s')\n\n",
+      DEFAULT_IMPORTER);
 
 #ifdef WITH_OSS
     fprintf(fd, "OSS device options:\n"
@@ -91,6 +106,11 @@ static void usage(FILE *fd)
       "  -j <name>      Create a JACK deck with the given name\n\n");
 #endif
 
+#ifdef WITH_ALSA
+    fprintf(fd, "MIDI control:\n"
+      "  -dicer <dev>   Novation Dicer\n\n");
+#endif
+
     fprintf(fd,
       "The ordering of options is important. Options apply to subsequent\n"
       "music libraries or decks, which can be given multiple times. See the\n"
@@ -103,18 +123,20 @@ static void usage(FILE *fd)
 
 int main(int argc, char *argv[])
 {
-    int r, n, decks;
+    int r, n, decks, priority;
     const char *importer, *scanner;
+    char *endptr;
+    size_t nctl;
     double speed;
     struct timecode_def *timecode;
-    bool protect;
+    bool protect, use_mlock;
 
     struct deck deck[3];
+    struct controller ctl[2];
     struct rt rt;
     struct library library;
 
 #if defined WITH_OSS || WITH_ALSA
-    char *endptr;
     int rate;
 #endif
 
@@ -128,7 +150,7 @@ int main(int argc, char *argv[])
 
     fprintf(stderr, "%s\n\n" NOTICE "\n\n", banner);
 
-    if (rt_global_init() == -1)
+    if (thread_global_init() == -1)
         return -1;
 
     if (rig_init() == -1)
@@ -137,11 +159,14 @@ int main(int argc, char *argv[])
     library_init(&library);
 
     decks = 0;
+    nctl = 0;
+    priority = DEFAULT_PRIORITY;
     importer = DEFAULT_IMPORTER;
     scanner = DEFAULT_SCANNER;
     timecode = NULL;
     speed = 1.0;
     protect = false;
+    use_mlock = false;
 
 #if defined WITH_OSS || WITH_ALSA
     rate = DEFAULT_RATE;
@@ -329,6 +354,11 @@ int main(int argc, char *argv[])
             if (r == -1)
                 return -1;
 
+            /* Connect this deck to available controllers */
+
+            for (n = 0; n < nctl; n++)
+                controller_add_deck(&ctl[n], &deck[decks]);
+
             decks++;
 
             argv += 2;
@@ -380,6 +410,36 @@ int main(int argc, char *argv[])
             argv++;
             argc--;
 
+        } else if (!strcmp(argv[0], "-k")) {
+
+            use_mlock = true;
+            track_use_mlock();
+
+            argv++;
+            argc--;
+
+        } else if (!strcmp(argv[0], "-q")) {
+
+            if (argc < 2) {
+                fprintf(stderr, "-q requires an integer argument.\n");
+                return -1;
+            }
+
+            priority = strtol(argv[1], &endptr, 10);
+            if (*endptr != '\0') {
+                fprintf(stderr, "-q requires an integer argument.\n");
+                return -1;
+            }
+
+            if (priority < 0) {
+                fprintf(stderr, "Priority (%d) must be zero or positive.\n",
+                        priority);
+                return -1;
+            }
+
+            argv += 2;
+            argc -= 2;
+
         } else if (!strcmp(argv[0], "-i")) {
 
             /* Importer script for subsequent decks */
@@ -434,6 +494,32 @@ int main(int argc, char *argv[])
             argv += 2;
             argc -= 2;
 
+#ifdef WITH_ALSA
+        } else if (!strcmp(argv[0], "-dicer")) {
+
+            struct controller *c;
+
+            if (nctl == sizeof ctl) {
+                fprintf(stderr, "Too many controllers; aborting.\n");
+                return -1;
+            }
+
+            c = &ctl[nctl];
+
+            if (argc < 2) {
+                fprintf(stderr, "Dicer requires an ALSA device name.\n");
+                return -1;
+            }
+
+            if (dicer_init(c, &rt, argv[1]) == -1)
+                return -1;
+
+            nctl++;
+
+            argv += 2;
+            argc -= 2;
+#endif
+
         } else {
             fprintf(stderr, "'%s' argument is unknown; try -h.\n", argv[0]);
             return -1;
@@ -450,9 +536,22 @@ int main(int argc, char *argv[])
         return -1;
     }
 
-    if (interface_start(deck, decks, &library) == -1)
+    for (n = 0; n < nctl; n++) {
+        if (rt_add_controller(&rt, &ctl[n]) == -1)
+            return -1;
+    }
+
+    /* Order is important: launch realtime thread first, then mlock */
+
+    if (rt_start(&rt, priority) == -1)
         return -1;
-    if (rt_start(&rt) == -1)
+
+    if (use_mlock && mlockall(MCL_CURRENT) == -1) {
+        perror("mlockall");
+        return -1;
+    }
+
+    if (interface_start(deck, decks, &library) == -1)
         return -1;
 
     if (rig_main() == -1)
@@ -460,16 +559,20 @@ int main(int argc, char *argv[])
 
     fprintf(stderr, "Exiting cleanly...\n");
 
-    rt_stop(&rt);
     interface_stop();
+    rt_stop(&rt);
 
     for (n = 0; n < decks; n++)
         deck_clear(&deck[n]);
 
+    for (n = 0; n < nctl; n++)
+        controller_clear(&ctl[n]);
+
     timecoder_free_lookup();
     library_clear(&library);
     rt_clear(&rt);
     rig_clear();
+    thread_global_clear();
 
     fprintf(stderr, "Done.\n");
 
